/*
       *  Licensed Materials - Property of IBM
       *  5725-G92 (C) Copyright IBM Corp. 2011, 2012. All Rights Reserved.
       *  US Government Users Restricted Rights - Use, duplication or
       *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

// Worklight comes with the jQuery framework bundled inside.
// If you do not want to use it, please comment out the line below.
window.$ = window.jQuery = WLJQ;

function wlCommonInit() {
	// Common initialization code goes here
	
	if ((WL.Client.getEnvironment() === 'iphone' ||
		WL.Client.getEnvironment() === 'ipad' ||
		WL.Client.getEnvironment() === 'android') &&
		typeof cordova !== 'undefined' &&
		typeof WL.JSONStore !== 'undefined' &&
		typeof WLJQ !== 'undefined') {

		//At this point all dependencies at met (valid environment, cordova, JSONStore and jQuery)

		//We perform operations like add and find on the usersCollection that
		//gets created when we initialize the collection.
		var usersCollection = null,

			//Cache variables in the DOM
			statusTag = WLJQ('div#status'),
			nameTag = WLJQ('input#name'),
			ageTag = WLJQ('input#age'),
			tableTag = WLJQ('table#user_table'),
			idTag = WLJQ('input#id'),
			nameFindTag = WLJQ('input#name_find'),

			//Logs messages to the console and to a div tag
			logMessage = function (msg) {
				WL.Logger.debug(msg);
				statusTag.text(msg);
				tableTag.html('');
			},

			//Simple function that check if obj is null or undefined
			checkUndefOrNull = function (obj) {
				return (typeof obj === 'undefined') || obj === null;
			},

			//Simple function that check if collection is init
			checkColInit = function (obj) {
				if ((typeof obj === 'undefined') || obj === null){
					logMessage('You must initialize the collection first.');
					return false;
				}else{ return true; }
			},

			//Generic callback called when there's a failure
			genericFailureCallback = function  (err) {
				logMessage(err + ' Error Message: ' + WL.JSONStore.getErrorMessage(err));
			};//end define vars


		//Convenience function to load the table with entries
		var reloadTable = function(results) {

				var msg = '<tr><td>ID</td><td>NAME</td><td>AGE</td></tr>',
					i = results.length,
					curr;

				if ( i < 1) {
	
					logMessage('No Entries Found');

				} else {

					while ( i-- ) {
						curr = results[i].json;
						id = results[i]._id;
						msg += '<tr><td>' + id + '</td><td>' + curr.name + '</td><td>' + curr.age + '</td></tr>';
					}
					
					WL.Logger.debug('Find Raw Results: ' + JSON.stringify(results));
					logMessage(''); //Clear Log
					tableTag.html(msg); //Add Table
				}

		};

		//Initializes the 'users' collection
		WLJQ('button#initCollection').bind('click', function () {
			
			WL.Logger.debug('Called button#initCollection');
			
			var collectionName = 'users',
				searchFields = {name: 'string', age: 'number'};

			var win =   function (data) {
				logMessage('Collection init successful: ' + data);
			};

			var options = {onSuccess: win, onFailure: genericFailureCallback};

			usersCollection = WL.JSONStore.initCollection(collectionName, searchFields, options);

		});

		//Adds a new user by his or her name and age
		WLJQ('button#add').bind('click', function () {

			WL.Logger.debug('Called button#store');

			var user_name = nameTag.val(),
				user_age = ageTag.val();

			if (user_name.length < 1 || user_age.length < 1 ||
				checkUndefOrNull(usersCollection)) {

				logMessage('You must init the collection and provide valid values');

			} else {

				var win =   function (data) {
					logMessage('Added ' + data + ' document.');
					nameTag.val('');
					ageTag.val('');
				};

				var options = {onSuccess: win, onFailure: genericFailureCallback};

				usersCollection.add({name: user_name, age: user_age},options);
			}
		});

		WLJQ('button#btn_remove').bind('click', function () {

			WL.Logger.debug('Called button#btn_remove');

			var name = nameTag.val(),
				id = idTag.val(),
				age = ageTag.val();

			var win =   function (data) {
				logMessage('Removed ' + data + ' document(s).' );
				nameTag.val('');
				ageTag.val('');
				idTag.val('');
			};

			var options = {onSuccess: win, onFailure: genericFailureCallback};

			if (age.length > 0 && name.length > 0) { //Remove specific document
				logMessage('Remove Document by name and age.' );
				usersCollection.remove({name: name, age: age}, options);

			} else if (name.length > 0 && age.length < 1) { //Remove All By Name
				logMessage('Remove All by Name.' );
				usersCollection.remove({name: name}, options);

			} else if (id.length > 0) { //Remove document by ID
				logMessage('Remove by ID.' + id );
				usersCollection.remove(parseInt(id), options);

			} else {
				logMessage('You must init the collection and provide valid values for name/age/id');
			}
		});

		WLJQ('button#find_name').bind('click', function () {

			WL.Logger.debug('Called button#find_name');

			if (!checkColInit(usersCollection)) {return;}

			var user_name = nameFindTag.val();

			if (user_name.length < 1) {
				logMessage('You must provide a search name');
			} else {

				var query = {name: user_name};

				var win =  function (data) {
					reloadTable(data);
					nameFindTag.val('');
				};

				var options = {onSuccess: win, onFailure: genericFailureCallback};

				usersCollection.find(query, options);
			}
		});


		WLJQ('button#find_all').bind('click', function () {

			WL.Logger.debug('Called button#find_all');

			if (!checkColInit(usersCollection)) {return;}

			var win =   function (data) {
				reloadTable(data);
			};

			var options = {onSuccess: win, onFailure: genericFailureCallback};

			usersCollection.findAll(options);
		});


		WLJQ('button#count').bind('click', function () {

			WL.Logger.debug('Called button#count');

			if (checkUndefOrNull(usersCollection)) {
				logMessage('You must init the collection first');
			} else {

				var win =   function (data) {
					logMessage('Documents stored: ' + data);
				};

				var options = {onSuccess: win, onFailure: genericFailureCallback};

				usersCollection.count(options);
			}
		});


		WLJQ('button#destroy').bind('click', function () {

			WL.Logger.debug('Called button#destroy');

			var win =   function () {
				logMessage('JSONStore was destroyed');
			};

			var options = {onSuccess: win, onFailure: genericFailureCallback};

			WL.JSONStore.destroy(options);
		});


		WLJQ('button#close').bind('click', function () {

			WL.Logger.debug('Called button#close');

			var win =   function () {
				logMessage('JSONStore was closed');
			};

			var options = {onSuccess: win, onFailure: genericFailureCallback};

			WL.JSONStore.closeAll(options);

		});

			
	}//end if statement to check for valid environments and dependencies

}//end wlCommonInit()